/* Jentchina -Obj
 * STRASSER Michel - 20040406		init
 * */

#include "JentchinaObj.h"
#include "../Central/pack_Central.h"
#include "../XmlReaders/pack_XmlReaders.h"
#include <glib.h>
#include <glib-object.h>

static GObjectClass *jnaObj_parentClass;

static void
impl_JentchinaObj_finalize(GObject *object)
{
	JentchinaObj *self=(JentchinaObj*)object;

	g_object_unref(self->cnl);

	jnaObj_parentClass->finalize(object);
}

static void
JentchinaObjClass_init (JentchinaObjClass *selfClass)
{
	GObjectClass *objectClass = (GObjectClass *)selfClass;
	jnaObj_parentClass = g_type_class_peek_parent (selfClass);

	objectClass->finalize=impl_JentchinaObj_finalize;
}

GType		JentchinaObj_getType		(void)
{
	static GType type = 0;

	if (!type) {
		GTypeInfo info = {
			sizeof (JentchinaObjClass),
			NULL, NULL,
			(GClassInitFunc) JentchinaObjClass_init,
			NULL, NULL,
			sizeof (JentchinaObj),
			0,
			NULL, NULL
		};

		type = g_type_register_static (
			G_TYPE_OBJECT,
			"JentchinaObj", &info, 0);
	}

	return type;
}

GObject* 	
JentchinaObj_new		(char* appName)
{
	JentchinaObj* retval;

	g_return_val_if_fail(appName, NULL);
	
	retval=g_object_new(JENTCHINA_TYPE_OBJ, NULL);
	retval->cnl=JnaCnlJentchina_new(appName);
	g_message(__PRETTY_FUNCTION__);

	return (GObject*) retval;
}

void 
JentchinaObj_print			(JentchinaObj* self)
{
	g_return_if_fail(JENTCHINA_IS_OBJ(self));

	JnaCnlJentchina_print(JNA_CNL_JENTCHINA(self->cnl));
}

GObject*	/* JentchinaTreeModel */
JentchinaObj_getNewModel			(JentchinaObj* self,
						 JentchinaFilterType filterType,
						 JentchinaTreeModelType modelType)
{
	GObject *retval;
	GObject* filtersTree;
	
	g_return_val_if_fail(JENTCHINA_IS_OBJ(self), NULL);
	g_return_val_if_fail(filterType>=0 && filterType<JentchinaFilterType_NUM, NULL);
	g_return_val_if_fail(modelType>=0 && modelType<JentchinaTreeModelType_NUM, NULL);

	filtersTree=JNA_CNL_JENTCHINA(self->cnl)->filtersTree[filterType];
	
	switch(modelType) {
		case JentchinaTreeModelType_SERVER:
			retval=JnaCnlServerTreeModel_new(filtersTree);
		default:
			g_assert("This can't happen");
	}

	return retval;
}

JentchinaStatus
JentchinaObj_getRendezVous			(JentchinaObj* self,
						 JentchinaIter in_iter,
						 char* in_user,
						 char* in_password,
						 LsmTree* in_doc,
						 GList** out_interfaces,	/* newly allocated, list of string */
						 int* out_vortexLifeTime,	/* not supported yet */
						 char** out_sessionKey)		/* newly allocated */
{
	JentchinaStatus retval=JentchinaStatus_OK;
	g_return_val_if_fail(JENTCHINA_IS_OBJ(self), JentchinaStatus_BAD_OBJ); 

	retval=JnaCnlJentchina_getRendezVous(JNA_CNL_JENTCHINA(self->cnl), in_iter, in_user, in_password, in_doc, out_interfaces,
			out_vortexLifeTime, out_sessionKey);

	return retval;
}

JentchinaStatus
JentchinaObj_applyFilter			(JentchinaObj* self,
						 char* in_sessionKey,
						 char* in_interfacesStream,
						 LsmTree** out_filteredDoc)	/* newly allocated */
{
	JentchinaStatus retval=JentchinaStatus_OK;
	g_return_val_if_fail(JENTCHINA_IS_OBJ(self), JentchinaStatus_BAD_OBJ); 

	retval=JnaCnlJentchina_applyFilter(JNA_CNL_JENTCHINA(self->cnl), in_sessionKey, in_interfacesStream, out_filteredDoc);
		
	return retval;
}

static void
impl_JentchinaObj_search			(JentchinaObj* self,
						 JentchinaIter in_iter,
						 gboolean* out_isFilter,
						 char** out_filterName,	/* Not newly allocated */
						 char** out_ressourcePath, /* like /Export/HTML/Ocean/TblFd-Ocean.xml, newly allocated */
						 char** out_picRessourcePath, /* newly allocated */
						 char** out_serverDomain,	/* Not newly allocated */
						 char** out_serverId,		/* Not newly allocated */
						 char** out_serverName)	/* Not newly allocated */

{
	GObject* filtersTree;
	int t, s, n, i;
	GObject* serverFiltersTree;
	*out_isFilter=FALSE;
	

	JnaCnlIter_getAllFast(in_iter, &t, &s, &n);
	filtersTree=JNA_CNL_JENTCHINA(self->cnl)->filtersTree[t];
	g_return_if_fail(s<JNA_CNL_FILTERS_TREE(filtersTree)->nServers);

	serverFiltersTree=JNA_CNL_FILTERS_TREE(filtersTree)->first;
	for(i=0 ; i<s ; i++)
		serverFiltersTree=JNA_CNL_SERVER_FILTERS_TREE(serverFiltersTree)->next;

	JnaCnlServerFiltersTree_getServerInfo(JNA_CNL_SERVER_FILTERS_TREE(serverFiltersTree), out_serverDomain, out_serverId, 
			out_serverName);
	g_return_if_fail(n<JNA_CNL_SERVER_FILTERS_TREE(serverFiltersTree)->max);

	if(n) {
		*out_isFilter=!JNA_CNL_SERVER_FILTERS_TREE(serverFiltersTree)->array[n].child;
		if(*out_isFilter) {
			int cur;
			GString* gstr;
			JnaCnlFilter* array=JNA_CNL_SERVER_FILTERS_TREE(serverFiltersTree)->array;
		
			*out_filterName=array[n].name;
			gstr=g_string_new("");
			for(cur=n ; cur ; cur=array[cur].parent) {
				g_string_prepend(gstr, array[cur].name);
				g_string_prepend_c(gstr, '/');
			}
			g_string_prepend(gstr, ((t==JentchinaFilterType_IMPORT)?"/Import":"/Export"));	
			*out_ressourcePath=g_strconcat(gstr->str, "/TblFd-", array[n].name, ".xml", NULL);
			*out_picRessourcePath=g_strconcat(gstr->str, "/TblFp-", array[n].name, ".png", NULL);
			g_string_free(gstr, TRUE);
		}
	}
}

static void
impl_JentchinaObj_searchAuthor			(JentchinaObj* self,
						 JentchinaIter in_iter,
						 char* in_id,
						 char** out_serverDomain,	/* Not newly allocated */
						 char** out_serverId,		/* Not newly allocated */
						 char** out_ressourcePath,	/* Newly allocated */
						 char** out_picRessourcePath)	/* Newly allocated */

{
	GObject* filtersTree;
	int t, s, n, i;
	GObject* serverFiltersTree;
	char* serverName;

	JnaCnlIter_getAllFast(in_iter, &t, &s, &n);
	filtersTree=JNA_CNL_JENTCHINA(self->cnl)->filtersTree[t];
	g_return_if_fail(s<JNA_CNL_FILTERS_TREE(filtersTree)->nServers);

	serverFiltersTree=JNA_CNL_FILTERS_TREE(filtersTree)->first;
	for(i=0 ; i<s ; i++)
		serverFiltersTree=JNA_CNL_SERVER_FILTERS_TREE(serverFiltersTree)->next;

	JnaCnlServerFiltersTree_getServerInfo(JNA_CNL_SERVER_FILTERS_TREE(serverFiltersTree), out_serverDomain, out_serverId, 
			&serverName);
	g_return_if_fail(n<JNA_CNL_SERVER_FILTERS_TREE(serverFiltersTree)->max);

	*out_ressourcePath=g_strconcat("/Authors/TblAi-", in_id, ".xml", NULL);
	*out_picRessourcePath=g_strconcat("/Authors/TblAp-", in_id, ".png", NULL);
}

JentchinaStatus
JentchinaObj_getInfos				(JentchinaObj* self,
						 JentchinaIter in_iter,
						 char** out_serverName,		/* newly allocated */ 
						 char** out_serverPic,		/* newly allocated */
						 char** out_filterName,		/* newly allocated */
						 int* out_version,
						 char** out_year,		/* newly allocated */
						 char** out_license,		/* newly allocated */
						 char** out_application,	/* newly allocated */
						 char** out_info,		/* newly allocated */
						 GList** out_authors,		/* newly allocated, list of string */
						 char** out_filterPic)		/* newly allocated */
{
	JentchinaStatus retval=JentchinaStatus_OK;
	gboolean isFilter;
	char* ressourcePath;
	char* ressourceLocal;
	char* picRessourcePath;
	char* serverDomain;
	char* serverId;
	char* serverName;
	char* filterName;

	*out_version=-1;
	*out_year=NULL;
	*out_license=NULL;
	*out_application=NULL;
	*out_info=NULL;
	*out_authors=NULL;


	g_return_val_if_fail(JENTCHINA_IS_OBJ(self), JentchinaStatus_BAD_OBJ); 
	
	JnaCnlIter_print(in_iter);

	impl_JentchinaObj_search(self, in_iter, &isFilter, &filterName, &ressourcePath, 
			&picRessourcePath, &serverDomain, &serverId, &serverName);

	*out_serverName=g_strdup(serverName);
	*out_serverPic=JnaCnlCacheAndBank_getRessource(
				JNA_CNL_CACHE_AND_BANK(JNA_CNL_JENTCHINA(self->cnl)->cacheAndBank),
					serverDomain,
					serverId,
					"/Server/TblSp-Server.png");
;
	if(isFilter) {
		*out_filterName=g_strdup(filterName);
		g_message("Is filter, ressource:'%s'", ressourcePath);
		ressourceLocal=JnaCnlCacheAndBank_getRessource(
				JNA_CNL_CACHE_AND_BANK(JNA_CNL_JENTCHINA(self->cnl)->cacheAndBank),
					serverDomain,
					serverId,
					ressourcePath);
		if(ressourceLocal) {
			GList* l;
			GObject* filterDesc; /* JnaXFilterDescriptionReader */
			
			filterDesc=JnaXFilterDescriptionReader_new(ressourceLocal);
			if(filterDesc) {
				*out_version=JNA_X_FILTER_DESCRIPTION_READER(filterDesc)->version;
				*out_year=g_strdup(JNA_X_FILTER_DESCRIPTION_READER(filterDesc)->year);
				*out_license=g_strdup(JNA_X_FILTER_DESCRIPTION_READER(filterDesc)->license);
				*out_application=g_strdup(JNA_X_FILTER_DESCRIPTION_READER(filterDesc)->application);
				*out_info=g_strdup(JNA_X_FILTER_DESCRIPTION_READER(filterDesc)->info);
				*out_authors=NULL;
				for(l=JNA_X_FILTER_DESCRIPTION_READER(filterDesc)->authors ; l ; l=g_list_next(l))
					*out_authors=g_list_append(*out_authors, g_strdup((char*)l->data));
				g_object_unref(G_OBJECT(filterDesc));
			}
#if 0
#endif
			// FIXME g_free(ressourceLocal);
		}

		*out_filterPic=JnaCnlCacheAndBank_getRessource(
				JNA_CNL_CACHE_AND_BANK(JNA_CNL_JENTCHINA(self->cnl)->cacheAndBank),
					serverDomain,
					serverId,
					picRessourcePath);

		g_free(ressourcePath);
		g_free(picRessourcePath);
	}
	else { /* something else, may be family or server */
		g_message("Is NOT filter");
		*out_filterName=NULL;
		*out_version=-1;
		*out_year=NULL;
		*out_license=NULL;
		*out_application=NULL;
		*out_info=NULL;
		*out_authors=NULL;
		*out_filterPic=NULL;
	}

	return retval;
}

JentchinaStatus
JentchinaObj_getAuthorInfos			(JentchinaObj* self,
						 JentchinaIter iter,
						 char* in_id,
						 char** out_name,		/* newly allocated */
						 char** out_email,		/* newly allocated */
						 char** out_picture)		/* newly allocated */
{
	JentchinaStatus retval=JentchinaStatus_OK;
	char* serverDomain;
	char* serverId;
	char* ressourcePath=NULL;
	char* ressourceLocal=NULL;
	char* picRessourcePath=NULL;
	GObject* authorInfoReader; /* JnaXAuthorInfoReader */

	*out_name=NULL;
	*out_email=NULL;
	*out_picture=NULL;

	impl_JentchinaObj_searchAuthor(self, iter, in_id, &serverDomain, &serverId, &ressourcePath, &picRessourcePath);

	*out_picture=JnaCnlCacheAndBank_getRessource(
					JNA_CNL_CACHE_AND_BANK(JNA_CNL_JENTCHINA(self->cnl)->cacheAndBank),
					serverDomain,
					serverId,
					picRessourcePath);

	ressourceLocal=JnaCnlCacheAndBank_getRessource(
					JNA_CNL_CACHE_AND_BANK(JNA_CNL_JENTCHINA(self->cnl)->cacheAndBank),
					serverDomain,
					serverId,
					ressourcePath);


	if(ressourceLocal) {
		authorInfoReader=JnaXAuthorInfoReader_new(ressourceLocal);
		if(authorInfoReader) {
			*out_name=g_strdup(JNA_X_AUTHOR_INFO_READER(authorInfoReader)->name);
			*out_email=g_strdup(JNA_X_AUTHOR_INFO_READER(authorInfoReader)->email);
			g_object_unref(authorInfoReader);
		}
		g_free(ressourceLocal);
	}

	if(ressourcePath)
		g_free(ressourcePath);
	if(picRessourcePath)
		g_free(picRessourcePath);

	return retval;
}

int
JentchinaObj_getData			(JentchinaObj* self,
					 JentchinaIter iter)
{
	int retval;	
	
	return retval;
}

void
JentchinaObj_setData			(JentchinaObj* self,
					 JentchinaIter iter,
					 int data)
{
}

