/* Jentchina -CnlCacheAndBank
 * STRASSER Michel - 20040329		init
 * */

#include <sys/types.h>	/* mkdir */
#include <sys/stat.h>	/* mkdir */

#include "JnaCnlCacheAndBank.h"
#include "JnaCnlStream.h"
#include "../XmlReaders/pack_XmlReaders.h"
#include "../Com/pack_JnaCom.h"
#include <glib.h>
#include <glib-object.h>

static GObjectClass *jnaCnlCacheAndBank_parentClass;

#define HELLO(num) \
	g_message("hello %d from %s at line %d", num, __PRETTY_FUNCTION__, __LINE__);

static void
impl_JnaCnlCacheAndBank_finalize(GObject *object)
{
	JnaCnlCacheAndBank *self=(JnaCnlCacheAndBank*)object;
	
	g_hash_table_destroy(self->coms);
	g_free(self->appName);
	g_free(self->cacheDir);

	jnaCnlCacheAndBank_parentClass->finalize(object);
}

static void
JnaCnlCacheAndBankClass_init (JnaCnlCacheAndBankClass *selfClass)
{
	GObjectClass *objectClass = (GObjectClass *)selfClass;
	jnaCnlCacheAndBank_parentClass = g_type_class_peek_parent (selfClass);

	objectClass->finalize=impl_JnaCnlCacheAndBank_finalize;
}

GType		JnaCnlCacheAndBank_getType		(void)
{
	static GType type = 0;

	if (!type) {
		GTypeInfo info = {
			sizeof (JnaCnlCacheAndBankClass),
			NULL, NULL,
			(GClassInitFunc) JnaCnlCacheAndBankClass_init,
			NULL, NULL,
			sizeof (JnaCnlCacheAndBank),
			0,
			NULL, NULL
		};

		type = g_type_register_static (
			G_TYPE_OBJECT,
			"JnaCnlCacheAndBank", &info, 0);
	}

	return type;
}

GObject* 	
JnaCnlCacheAndBank_new		(char* appName)
{
	JnaCnlCacheAndBank* retval;
	char* jentchinaDir;
	char* cache;

	g_return_val_if_fail(appName, NULL);
	
	retval=g_object_new(JNA_TYPE_CNL_CACHE_AND_BANK, NULL);
	
	retval->appName=g_strdup(appName);
	retval->coms=g_hash_table_new_full(g_str_hash, g_str_equal, (GDestroyNotify)g_free, (GDestroyNotify)g_object_unref);
#ifdef G_OS_WIN32
#define JnaCnlCacheAndBank_JENTCHINA_DIR "JentchinaRunTimeFiles"
#else
#define JnaCnlCacheAndBank_JENTCHINA_DIR ".Jentchina"
#endif

	jentchinaDir=g_build_filename(g_get_home_dir(), JnaCnlCacheAndBank_JENTCHINA_DIR, NULL);
	if(g_file_test(jentchinaDir, G_FILE_TEST_EXISTS)) {
		if(!g_file_test(jentchinaDir, G_FILE_TEST_IS_DIR)) 
			g_error("%s Jentchina working file exists but it isn't a directory, "
					"unrecoverable error, delete this fucking file and try again !", jentchinaDir);
	}
	else {
		g_return_val_if_fail(!mkdir(jentchinaDir, 00777), NULL);
	}
		
	cache=g_build_filename(jentchinaDir, "Cache", NULL);
	g_free(jentchinaDir);
	
	if(g_file_test(cache, G_FILE_TEST_EXISTS)) {
		if(!g_file_test(cache, G_FILE_TEST_IS_DIR)) 
			g_error("%s Jentchina cache file exists but it isn't a directory, "
					"unrecoverable error, delete this fucking file and try again !", cache);
	}
	else {
		g_return_val_if_fail(!mkdir(cache, 00777), NULL);
	}

	retval->cacheDir=g_build_filename(cache, appName, NULL);
	g_free(cache);
	
	if(g_file_test(retval->cacheDir, G_FILE_TEST_EXISTS)) {
		if(!g_file_test(retval->cacheDir, G_FILE_TEST_IS_DIR)) 
			g_error("%s Jentchina cache file exists but it isn't a directory, "
					"unrecoverable error, delete this fucking file and try again !", retval->cacheDir);
	}
	else {
		g_return_val_if_fail(!mkdir(retval->cacheDir, 00777), NULL);
	}

	return (GObject*) retval;
}

/* returns FALSE if already registered */
gboolean
JnaCnlCacheAndBank_registerServer	(JnaCnlCacheAndBank* self,
					 GObject* com,
					 GObject* server, 	/* JnaXServerInfoReader */
					 GObject* stream)	/* JnaCnlStream */
{
	char* str;
	gboolean retval;
	
	g_return_val_if_fail(JNA_IS_CNL_CACHE_AND_BANK(self), FALSE);
	g_return_val_if_fail(JNA_IS_X_SERVER_INFO_READER(server), FALSE);
	g_return_val_if_fail(JNA_IS_CNL_STREAM(stream), FALSE);

	str=g_strconcat(JNA_X_SERVER_INFO_READER(server)->domain, "::", JNA_X_SERVER_INFO_READER(server)->id, NULL);

	if(g_hash_table_steal(self->coms,str)) {
		g_message("server '%s' already registered, may be replication", str);
		g_free(str);
		retval=FALSE;
	}
	else {
		char* domain;
		char* id;
		char* serverS;
		char* file;

		g_object_ref(com);
		g_hash_table_insert(self->coms, str, com);
		retval=TRUE;
		domain=g_build_filename(self->cacheDir, JNA_X_SERVER_INFO_READER(server)->domain, NULL);
		id=g_build_filename(domain, JNA_X_SERVER_INFO_READER(server)->id, NULL);
		serverS=g_build_filename(id, "Server", NULL);
		file=g_build_filename(serverS, "TblSi-Server.xml", NULL);
	
#define JnaCnlCacheAndBank_CREATE_DIR(dir) 				\
		if(g_file_test(dir, G_FILE_TEST_EXISTS)) { 		\
			if(!g_file_test(dir, G_FILE_TEST_IS_DIR)) 	\
				g_error("%s file exists but it isn't a directory, " \
						"unrecoverable error, delete this fucking file and try again !", dir);	\
		}							\
		else {							\
			g_return_val_if_fail(!mkdir(dir, 00777), FALSE);\
		}							\
									\
		g_free(dir);

		JnaCnlCacheAndBank_CREATE_DIR(domain);
		JnaCnlCacheAndBank_CREATE_DIR(id);
		JnaCnlCacheAndBank_CREATE_DIR(serverS);

		JnaCnlStream_writeIntoFile(JNA_CNL_STREAM(stream), file);
		g_free(file);
	}
	
	return retval;
}

/* returns NULL if not registered */
GObject*
JnaCnlCacheAndBank_getCom		(JnaCnlCacheAndBank* self,
					 char* domain,
					 char* id)
{
	GObject* retval;
	char* str;

	g_return_val_if_fail(JNA_IS_CNL_CACHE_AND_BANK(self), NULL);
	g_return_val_if_fail(domain, NULL);
	g_return_val_if_fail(id, NULL);

	str=g_strconcat(domain, "::", id, NULL);

	retval=g_hash_table_lookup(self->coms,str);	

	g_free(str);
	
	return retval;
}

static void
impl_JnaCnlCacheAndBank_createDirs	(JnaCnlCacheAndBank* self,
					 char* serverDomain,
					 char* serverId,
					 char** splitted)
{
	char *dir1, *dir2;
	int num, i;

	dir1=g_build_filename(self->cacheDir, serverDomain, serverId, NULL);
	
	g_return_if_fail(splitted[0]); /* this should not happen */
	
	for(num=0 ; splitted[num] ; num++)	/* counting */
		;
	
	num--;
	for(i=0 ; i<num ; i++) {
		dir2=g_strconcat(dir1, G_DIR_SEPARATOR_S, splitted[i], NULL);
		g_free(dir1);
		
		if(g_file_test(dir2, G_FILE_TEST_EXISTS)) { 		\
			if(!g_file_test(dir2, G_FILE_TEST_IS_DIR)) 	\
				g_error("%s file exists but it isn't a directory, " \
						"unrecoverable error, delete this fucking file and try again !", dir2);	\
		}							\
		else {							\
			g_return_if_fail(!mkdir(dir2, 00777));
		}							\
	
		dir1=dir2;
	}
		
	g_free(dir1);
}

char*	/* newly allocated */
JnaCnlCacheAndBank_getRessource		(JnaCnlCacheAndBank* self,
					 char* serverDomain,
					 char* serverId,
					 char* ressource)
{
	char* retval;
	char** splitted;
	char* joined;

	g_return_val_if_fail(JNA_IS_CNL_CACHE_AND_BANK(self), NULL);
	g_return_val_if_fail(serverDomain, NULL);
	g_return_val_if_fail(serverId, NULL);
	g_return_val_if_fail(ressource, NULL);

	splitted=g_strsplit(ressource, "/", 0);
	joined=g_strjoinv(G_DIR_SEPARATOR_S, splitted);
	retval=g_build_filename(self->cacheDir, serverDomain, serverId, joined, NULL);

	impl_JnaCnlCacheAndBank_createDirs(self, serverDomain, serverId, splitted);
	g_free(joined);
	g_strfreev(splitted);
	
	if(!g_file_test(retval, G_FILE_TEST_EXISTS | G_FILE_TEST_IS_REGULAR)) {
		char* str;
		GObject* com;
		GObject* stream;
		GObject* comException=NULL;

		str=g_strconcat(serverDomain, "::", serverId, NULL);

		com=g_hash_table_lookup(self->coms, str);
		g_return_val_if_fail(JNA_IS_COM(com), NULL);

		stream=JnaCom_getRessource(JNA_COM(com), ressource, &comException);

		if(comException) {
			char* problem;

			g_object_ref(comException);
			problem=JnaComException_toString(JNA_COM_EXCEPTION(comException));

			g_critical("Got the following problem : '%s' while trying to get ressource '%s'" 
					"on machine identified by domain::id='%s'", problem, ressource, str);
			
			g_free(problem);
			g_object_unref(comException);
			
			return NULL;
		}

		g_free(str);

		JnaCnlStream_writeIntoFile(JNA_CNL_STREAM(stream), retval);
	}
	
	return retval;
}


