/* Jentchina -CnlJentchina
 * STRASSER Michel - 20040206		init
 * */


#include "JnaCnlJentchina.h"
#include "JnaCnlFiltersTree.h"
#include "JnaCnlServerFiltersTree.h"
#include "../Com/pack_JnaCom.h"
#include "JnaCnlIter.h"
#include "JnaCnlStream.h"
#include "JnaCnlCacheAndBank.h"
#include "../Grabber/pack_Grabber.h"
#include <glib.h>
#include <glib-object.h>
#include <string.h>
#include <lsmlib.h>

static GObjectClass *jnaCnlJentchina_parentClass;

static void
impl_JnaCnlJentchina_finalize(GObject *object)
{
	JnaCnlJentchina *self=(JnaCnlJentchina*)object;
	int i;

	for(i=0 ; i < JnaFilterType_NUM ; i++)
		g_object_unref(self->filtersTree[i]);

	g_object_unref(self->grabber);
	g_object_unref(self->cacheAndBank);
	g_free(self->appName);
	
	jnaCnlJentchina_parentClass->finalize(object);
}

static void
JnaCnlJentchinaClass_init (JnaCnlJentchinaClass *selfClass)
{
	GObjectClass *objectClass = (GObjectClass *)selfClass;
	jnaCnlJentchina_parentClass = g_type_class_peek_parent (selfClass);

	objectClass->finalize=impl_JnaCnlJentchina_finalize;
}

GType		JnaCnlJentchina_getType		(void)
{
	static GType type = 0;

	if (!type) {
		GTypeInfo info = {
			sizeof (JnaCnlJentchinaClass),
			NULL, NULL,
			(GClassInitFunc) JnaCnlJentchinaClass_init,
			NULL, NULL,
			sizeof (JnaCnlJentchina),
			0,
			NULL, NULL
		};

		type = g_type_register_static (
			G_TYPE_OBJECT,
			"JnaCnlJentchina", &info, 0);
	}

	return type;
}

GObject* 	
JnaCnlJentchina_new		(char* appName)
{
	JnaCnlJentchina* retval;
	int i;

	g_return_val_if_fail(appName, NULL);
	
	g_message(__PRETTY_FUNCTION__);
	retval=g_object_new(JNA_TYPE_CNL_JENTCHINA, NULL);

	for(i=0 ; i < JnaFilterType_NUM ; i++)
		retval->filtersTree[i]=JnaCnlFiltersTree_new((JnaFilterType)i);

	retval->cacheAndBank=JnaCnlCacheAndBank_new(appName);
	retval->appName=g_strdup(appName);
#define JnaCnlJentchina_MAX_N_AGENTS 10
	retval->grabber=JnaGrabber_new(G_OBJECT(retval), JnaCnlJentchina_MAX_N_AGENTS, appName); 
#undef JnaCnlJentchina_MAX_N_AGENTS
	
	return (GObject*) retval;
}

JnaCnlStatus
JnaCnlJentchina_getRendezVous	(JnaCnlJentchina* self,
				 JnaCnlIter iter,
				 char* user,
				 char* password,
				 LsmTree* doc,
				 GList** out_interfacesList, // newly allocated
				 int* out_vortexLifeTime,
				 char** out_sessionKey) //newly allocated
{
	JnaCnlStatus retval=JnaCnlStatus_OK;
	GObject* filtersTree;
	int t, s, n, i;
	GObject* serverFiltersTree;
	gboolean isFilter=FALSE;
	char *serverDomain, *serverId, *serverName;

	*out_interfacesList=NULL;
	*out_vortexLifeTime=-1;
	*out_sessionKey=NULL;

	g_return_val_if_fail(JNA_IS_CNL_JENTCHINA(self), JnaCnlStatus_INVALID_OBJ);

	JnaCnlIter_getAllFast(iter, &t, &s, &n);
	filtersTree=self->filtersTree[t];
	g_return_val_if_fail(s<JNA_CNL_FILTERS_TREE(filtersTree)->nServers, JnaCnlStatus_INVALID_ITER);

	serverFiltersTree=JNA_CNL_FILTERS_TREE(filtersTree)->first;
	for(i=0 ; i<s ; i++)
		serverFiltersTree=JNA_CNL_SERVER_FILTERS_TREE(serverFiltersTree)->next;

	JnaCnlServerFiltersTree_getServerInfo(JNA_CNL_SERVER_FILTERS_TREE(serverFiltersTree), &serverDomain, &serverId, &serverName);
	g_return_val_if_fail(n<JNA_CNL_SERVER_FILTERS_TREE(serverFiltersTree)->max, JnaCnlStatus_INVALID_ITER);

	if(n) {
		int cur;
		GString* gstr;
		JnaCnlFilter* array=JNA_CNL_SERVER_FILTERS_TREE(serverFiltersTree)->array;
		GObject* com;	/* JnaCom */
		GObject* comException=NULL; /* JnaComException */
		GObject* rendezVous; /* JnaComRendezVous */
		char* lsmStreamS;
		GObject* documentJnaCnlStream; // JnaCnlStream
		GList* l;
	
		isFilter=!JNA_CNL_SERVER_FILTERS_TREE(array[n].child);
		g_return_val_if_fail(isFilter, JnaCnlStatus_INVALID_ITER);
					
		gstr=g_string_new("");
		for(cur=n ; cur ; cur=array[cur].parent) {
			g_string_prepend(gstr, array[cur].name);
			g_string_prepend_c(gstr, '/');
		}

		com=JnaCnlCacheAndBank_getCom(JNA_CNL_CACHE_AND_BANK(self->cacheAndBank), serverDomain, serverId);
		g_return_val_if_fail(JNA_IS_COM(com), JnaCnlStatus_INVALID_ITER);
		g_object_ref(com);
		
		LsmTree_toLsmXmlStream(doc, &lsmStreamS);
		documentJnaCnlStream=JnaCnlStream_new(lsmStreamS, strlen(lsmStreamS));
	
		rendezVous=JnaCom_getRendezVous(JNA_COM(com), (JnaComFilterType)t, gstr->str, user, password, 
				documentJnaCnlStream, &comException);
		if(comException)
			JnaComException_print(JNA_COM_EXCEPTION(comException));
		g_return_val_if_fail(!comException, JnaCnlStatus_INVALID_ITER);
		g_return_val_if_fail(JNA_IS_COM_RENDEZ_VOUS(rendezVous), JnaCnlStatus_INVALID_ITER);
		g_object_unref(com);

		*out_interfacesList=NULL;
		for(l=JNA_COM_RENDEZ_VOUS(rendezVous)->interfacesList ; l ; l=g_list_next(l)) {
			char* rStr=l->data;
			char* local;

			local=JnaCnlCacheAndBank_getRessource(JNA_CNL_CACHE_AND_BANK(self->cacheAndBank), serverDomain, serverId, rStr);
		
			g_message("local is %s", local);
			if(local)
				*out_interfacesList=g_list_append(*out_interfacesList, local);
			else
				; /* Case to be specified */
		}
		*out_sessionKey=g_strjoin("::", serverDomain, serverId, JNA_COM_RENDEZ_VOUS(rendezVous)->sessionKey, NULL);;
		*out_vortexLifeTime=JNA_COM_RENDEZ_VOUS(rendezVous)->vortexLifeTime;
		g_string_free(gstr, TRUE);
		g_object_unref(rendezVous);
		g_object_unref(documentJnaCnlStream);
		g_free(lsmStreamS);
	}
	else
		return JnaCnlStatus_INVALID_ITER;

	return retval;
}
				 
JnaCnlStatus
JnaCnlJentchina_applyFilter	(JnaCnlJentchina* self,
				 char* sessionKey,
				 char* interfaces,
				 LsmTree** out_filteredDoc)	//newly allocated
{
	JnaCnlStatus retval=JnaCnlStatus_OK;
	char** splitted;
	int i;
	GObject* com;
	GObject* interfacesJnaCnlStream; // JnaCnlStream
	GObject* comException=NULL; /* JnaComException */
	GObject* stream; /* JnaCnlStream */
	
	g_return_val_if_fail(JNA_IS_CNL_JENTCHINA(self), JnaCnlStatus_INVALID_OBJ);
	g_return_val_if_fail(sessionKey, JnaCnlStatus_INVALID_SESSION_KEY);
	splitted=g_strsplit(sessionKey, "::", 0);

	for(i=0 ; i<3 ; i++)
		g_return_val_if_fail(splitted[i], JnaCnlStatus_INVALID_SESSION_KEY);

	com=JnaCnlCacheAndBank_getCom(JNA_CNL_CACHE_AND_BANK(self->cacheAndBank), splitted[0], splitted[1]);
	g_return_val_if_fail(JNA_IS_COM(com), JnaCnlStatus_INVALID_SESSION_KEY);

	interfacesJnaCnlStream=JnaCnlStream_new(interfaces, strlen(interfaces));
	stream=JnaCom_applyFilter(JNA_COM(com), splitted[2], interfacesJnaCnlStream, &comException); 
	g_object_unref(interfacesJnaCnlStream);
	if(comException)
		JnaComException_print(JNA_COM_EXCEPTION(comException));
	g_return_val_if_fail(!comException, JnaCnlStatus_APPLY_FILTER_ERROR);
	g_strfreev(splitted);


	*out_filteredDoc=LsmTree_newFromLsmXmlStream(JNA_CNL_STREAM(stream)->seq);
	g_object_unref(stream);
	/*TODO */
	return retval;
}

void
JnaCnlJentchina_print		(JnaCnlJentchina* self)
{
	g_return_if_fail(JNA_IS_CNL_JENTCHINA(self));

	g_print("Jentchina::Central::Jentchina .filtersTree[JnaFilterType_IMPORT]={ \n");
	JnaCnlFiltersTree_print(JNA_CNL_FILTERS_TREE(self->filtersTree[JnaFilterType_IMPORT]));
	g_print("}\n .filtersTree[JnaFilterType_EXPORT]={ \n");
	JnaCnlFiltersTree_print(JNA_CNL_FILTERS_TREE(self->filtersTree[JnaFilterType_EXPORT]));
	g_print("}\n"); /* to be continued */
}

