/* Jentchina - Grabber 
 * STRASSER Michel - 20040404		init (simple sequential version)
 * STRASSER Michel - 20040508		adding search of servers list using gnome-vfs
 * */


#include "JnaGrabber.h"
#include "JnaGrbURL.h"
#include "../Central/pack_Central.h"
#include "../XmlReaders/pack_XmlReaders.h"
#include "../Com/pack_JnaCom.h"
#include <glib.h>
#include <glib-object.h>

static GObjectClass *jnaGrabber_parentClass;

static void
impl_JnaGrabber_finalize(GObject *object)
{
	JnaGrabber *self=(JnaGrabber*)object;

	g_free(self->appName);
	g_object_unref(self->serversListReader);
	g_object_unref(self->comFactory);

	g_hash_table_destroy(self->checkSumHash);
	
	jnaGrabber_parentClass->finalize(object);
}

static void
JnaGrabberClass_init (JnaGrabberClass *selfClass)
{
	GObjectClass *objectClass = (GObjectClass *)selfClass;
	jnaGrabber_parentClass = g_type_class_peek_parent (selfClass);

	objectClass->finalize=impl_JnaGrabber_finalize;
}

GType		JnaGrabber_getType		(void)
{
	static GType type = 0;

	if (!type) {
		GTypeInfo info = {
			sizeof (JnaGrabberClass),
			NULL, NULL,
			(GClassInitFunc) JnaGrabberClass_init,
			NULL, NULL,
			sizeof (JnaGrabber),
			0,
			NULL, NULL
		};

		type = g_type_register_static (
			G_TYPE_OBJECT,
			"JnaGrabber", &info, 0);
	}

	return type;
}

static gboolean 	/* returns FALSE if there was an error or already registered*/
impl_JnaGrabber_registerServer				(JnaGrabber* self,
							 GObject* xServer,
							 GObject** out_serverInfo)
{
	gboolean retval=TRUE;
	GObject* com;
	GObject* serverInfoStream;
	GObject* comException;
	GObject* serverInfo;

	g_return_val_if_fail(JNA_IS_X_SERVER(xServer), FALSE);

	g_message("%s, %d getNewComFromXServer", __PRETTY_FUNCTION__, __LINE__);
	com=JnaComFactory_getNewComFromXServer(JNA_COM_FACTORY(self->comFactory), xServer);
	g_message("%s, %d getNewComFromXServer made", __PRETTY_FUNCTION__, __LINE__);
	if(!com) {
		char* xServerS;

		xServerS=JnaXServer_toString(JNA_X_SERVER(xServer));
		g_critical("Couldn't convert JnaXServer '%s' description in a valid JnaCom communication descriptor", xServerS);
		g_free(xServerS);
		
		return FALSE;
	}
	
	comException=NULL;
	g_message("%s, %d getRessource /Server/TblSi-Server.xml", __PRETTY_FUNCTION__, __LINE__);
	serverInfoStream=JnaCom_getRessource(JNA_COM(com), "/Server/TblSi-Server.xml", &comException);
	g_message("%s, %d getRessource /Server/TblSi-Server.xml made", __PRETTY_FUNCTION__, __LINE__);
	if(comException) {
		char* xServerS;
		char* problem;

		g_object_ref(comException);
		xServerS=JnaXServer_toString(JNA_X_SERVER(xServer));
		problem=JnaComException_toString(JNA_COM_EXCEPTION(comException));

		g_critical("Got the following problem : '%s' while trying to get JnaXServer '%s' identifiers", problem, xServerS);
		
		g_free(xServerS);
		g_free(problem);
		g_object_unref(comException);
		
		return FALSE;
	}
	if(!serverInfoStream) {
		/* This should not happen */
		g_critical("Got no exception but serverStream is NULL");
	}

	serverInfo=JnaXServerInfoReader_newFromStream(serverInfoStream);
	if(!serverInfo) {
		g_critical("Not valid server info stream");
		return FALSE;
	}

	JnaXServerInfoReader_print(JNA_X_SERVER_INFO_READER(serverInfo));

	if(!JnaCnlCacheAndBank_registerServer(
				JNA_CNL_CACHE_AND_BANK(JNA_CNL_JENTCHINA(self->jentchina)->cacheAndBank),
				com,
				serverInfo,
				serverInfoStream))
		return FALSE;

	*out_serverInfo=serverInfo;
	
	g_object_unref(serverInfoStream);
	g_object_unref(com);

	g_message("%s, %d registering finished", __PRETTY_FUNCTION__, __LINE__);
	return retval;
}		

static gboolean 	/* returns FALSE if there was an error */
impl_JnaGrabber_getFiltersTreeAndMerge			(JnaGrabber* self,
							 GObject* serverInfo,
							 int filterType)	/* 0: import, 1: export */
{
	GObject* tree=NULL;
	char* file;
	gboolean retval=TRUE;
	
	g_return_val_if_fail((file=JnaCnlCacheAndBank_getRessource(
				JNA_CNL_CACHE_AND_BANK(JNA_CNL_JENTCHINA(self->jentchina)->cacheAndBank),
				JNA_X_SERVER_INFO_READER(serverInfo)->domain,
				JNA_X_SERVER_INFO_READER(serverInfo)->id,
					(filterType?"/Server/TblFt-Export.xml":"/Server/TblFt-Import.xml") )), FALSE);
	g_return_val_if_fail((tree=JnaXFiltersTreeReader_new(file)), FALSE);
	//JnaXFiltersTreeReader_print(JNA_X_FILTERS_TREE_READER(tree));
	JnaCnlFiltersTree_merge(JNA_CNL_FILTERS_TREE(JNA_CNL_JENTCHINA(self->jentchina)->filtersTree[filterType]), serverInfo, tree);
	//JnaCnlFiltersTree_print(JNA_CNL_FILTERS_TREE(JNA_CNL_JENTCHINA(self->jentchina)->filtersTree[filterType]));
		
	return retval;
}

static void
impl_JnaGrabber_contactServersAndGetTheirFeatures	(JnaGrabber* self,
							 GList* servers)
{
	GList* l;

	for(l=servers ; l ; l=g_list_next(l)) {
		GObject* serverInfo; /* JnaXServerInfoReader */
		
		if(!impl_JnaGrabber_registerServer(self, G_OBJECT(l->data), &serverInfo)) continue;
		if(!impl_JnaGrabber_getFiltersTreeAndMerge(self, serverInfo, 0)) continue;
		if(!impl_JnaGrabber_getFiltersTreeAndMerge(self, serverInfo, 1)) continue;

		g_object_unref(serverInfo);
	}

}

static void
impl_JnaGrabber_scanServerListRecursive	(JnaGrabber* self,
					 char* url,
					 int nRecurse)
{
	static int times=0;
	GObject* urlObj;
	int checkSumInt;

	g_message("%s, url:'%s', times:'%d', nRecurse:'%d'", __PRETTY_FUNCTION__, url, times, nRecurse);
	times++;

	urlObj=JnaGrbURL_new(url);
	g_return_if_fail(urlObj);
	g_print("URL stream:'%s', checkSum:'%u'\n", 
				JNA_GRB_URL(urlObj)->stream,
				(unsigned int)JNA_GRB_URL(urlObj)->checkSum);

	checkSumInt=(int)(JNA_GRB_URL(urlObj)->checkSum);
	if(!g_hash_table_lookup(self->checkSumHash, GINT_TO_POINTER(checkSumInt))) {
		GObject* listReader; /* JnaXServersListReader */
		
		g_hash_table_insert(self->checkSumHash, GINT_TO_POINTER(checkSumInt), GINT_TO_POINTER(TRUE));
		listReader=JnaXServersListReader_newFromStream(JNA_GRB_URL(urlObj)->stream);
		if(listReader) {
			GList* l;
			impl_JnaGrabber_contactServersAndGetTheirFeatures(self,JNA_X_SERVERS_LIST_READER(listReader)->servers);

			for(l=JNA_X_SERVERS_LIST_READER(listReader)->serversList; l; l=g_list_next(l)) 
				impl_JnaGrabber_scanServerListRecursive	(self, l->data, nRecurse+1);
			g_object_unref(listReader);
		}
	}
	else 
		g_message("Servers list at url='%s' has already been processed", url);
	
	
	g_object_unref(urlObj);
}

static char*
impl_JnaGrabber_firstUrl	(JnaGrabber* self,
				 char* appName)
{
	char* retval;
	char* file;
	char* fileFull;
	
	file=g_strconcat("TblSl-", self->appName, ".xml", NULL);
	fileFull=g_build_filename(TBL_VAR_DIR, "FiltersServersList", file, NULL);
	retval=g_strconcat("file://", fileFull, NULL);

	g_free(fileFull);
	g_free(file);

	return retval;
}

GObject* 	
JnaGrabber_new			(GObject* jentchina, // JnaCnlJentchina
				 int maxNAgents,
				 char* appName)
{
	JnaGrabber* retval;
	char *urlS;
	GObject* url;

	urlS=g_strconcat("file:///var/lib/TheBirdsLake/FiltersServersList/TblSl-", appName, ".xml", NULL);
	
	g_return_val_if_fail(appName, NULL);
	g_return_val_if_fail(JNA_IS_CNL_JENTCHINA(jentchina), NULL);
	
	g_message(__PRETTY_FUNCTION__);
	url=JnaGrbURL_new(urlS);
	if(url) {
		g_print("JnaGrbURL .url:'%s', .stream:'%s', .checkSum:'%u'\n", 
				urlS, 
				JNA_GRB_URL(url)->stream,
				(unsigned int)JNA_GRB_URL(url)->checkSum);
		g_object_unref(url);
	}
	retval=g_object_new(JNA_TYPE_GRABBER, NULL);

	retval->jentchina=jentchina;
	retval->appName=g_strdup(appName);
	retval->comFactory=JnaComFactory_new();
	retval->checkSumHash=g_hash_table_new(g_direct_hash, g_direct_equal);
	retval->maxRecurse=100;
		
	urlS=impl_JnaGrabber_firstUrl(retval, appName);

	impl_JnaGrabber_scanServerListRecursive	(retval, urlS, 0);
	g_free(urlS);


	return (GObject*) retval;
}

