/* Jentchina -XFilterDescriptionReader
 * STRASSER Michel - 20040303		init
 * */

#include "JnaXFilterDescriptionReader.h"
#include "JnaXCommon.h"
#include <glib.h>
#include <glib-object.h>
#include <libxml/parser.h>
#include <libxml/tree.h>
#include <libxml/xpath.h>
#include <string.h>
#include <stdlib.h> // atoi

static GObjectClass *jnaXFilterDescriptionReader_parentClass;

static void
impl_JnaXFilterDescriptionReader_finalize(GObject *object)
{
	GList* l;
	JnaXFilterDescriptionReader *self=(JnaXFilterDescriptionReader*)object;
	g_free(self->id);

	/* FIXME */
#if 0
	
	for(l=self->authors ; l ; l=g_list_next(l)) {
		g_free(G_OBJECT(l->data));
	}

#endif	
	g_list_free(self->authors);
	self->authors=NULL;
	
	g_free(self->year);
	g_free(self->license);
	g_free(self->application);
	g_free(self->info);
	jnaXFilterDescriptionReader_parentClass->finalize(object);
}

static void
JnaXFilterDescriptionReaderClass_init (JnaXFilterDescriptionReaderClass *selfClass)
{
	GObjectClass *objectClass = (GObjectClass *)selfClass;
	jnaXFilterDescriptionReader_parentClass = g_type_class_peek_parent (selfClass);

	objectClass->finalize=impl_JnaXFilterDescriptionReader_finalize;
}

char*
JnaXFilterDescriptionReader_toString		(JnaXFilterDescriptionReader* self)
{
	char* retval;
	GList* l;
	int length=0;
	int offset;
	char* authors;
	
	g_return_val_if_fail(JNA_IS_X_FILTER_DESCRIPTION_READER(self), NULL);

	length=3; // terminal '\0' character + strlen("()");
	for(l=self->authors ; l ; l=g_list_next(l)) {
		length+=2; // strlen(", "
		length+=strlen((char*)l->data);
	}
	
	authors=g_new(char, length);

	offset=0;
	authors[offset]='{';
	offset++;

	for(l=self->authors ; l ; l=g_list_next(l)) {
		int le;
		le=strlen((char*)l->data);
		g_memmove(authors+offset, (char*)l->data, le);
		offset+=le;
		authors[offset]=',';
		offset++;
		authors[offset]=' ';
		offset++;
	}
	
	authors[offset]='}';
	offset++;

	authors[offset]=0;
	
	retval=g_strdup_printf("Jna::XmlReaders::FilterDescriptionReader .id='%s' .authors=%s' .version='%d' "
			".year='%s' .license='%s' .application='%s' .info='%s'", 
			self->id, 
			authors,
			self->version,
			self->year,
			self->license,
			self->application,
			self->info);

	g_free(authors);
	
	return retval;
}

void
JnaXFilterDescriptionReader_print		(JnaXFilterDescriptionReader* self)
{
	char* str;
	
	g_return_if_fail(JNA_IS_X_FILTER_DESCRIPTION_READER(self));
	
	str=JnaXFilterDescriptionReader_toString(self);
	g_print(str);
	g_free(str);
}

GType		JnaXFilterDescriptionReader_getType		(void)
{
	static GType type = 0;

	if (!type) {
		GTypeInfo info = {
			sizeof (JnaXFilterDescriptionReaderClass),
			NULL, NULL,
			(GClassInitFunc) JnaXFilterDescriptionReaderClass_init,
			NULL, NULL,
			sizeof (JnaXFilterDescriptionReader),
			0,
			NULL, NULL
		};

		type = g_type_register_static (
			G_TYPE_OBJECT,
			"JnaXFilterDescriptionReader", &info, 0);
	}

	return type;
}

GObject* 	
JnaXFilterDescriptionReader_new			(char* xmlFile)
{
	JnaXFilterDescriptionReader* retval;
	xmlDocPtr doc;
	xmlXPathContextPtr xpath_context;
	xmlXPathObjectPtr xpath_object=NULL;
	xmlNodePtr node;
	int i, max;
	char *version;

	g_return_val_if_fail(xmlFile, NULL);
	
	doc=xmlParseFile(xmlFile);

	if(!doc) {
		g_critical("File %s is not a valid xml file", xmlFile);
		return NULL;
	}

	retval=g_object_new(JNA_TYPE_X_FILTER_DESCRIPTION_READER, NULL);

	retval->id=JnaXCommon_getRadical(xmlFile);
	g_return_val_if_fail(retval->id, NULL);

	xpath_context = xmlXPathNewContext (doc);

	xpath_object = xmlXPathEval ("/TblFd/TblFdAuthors/TblFdAuthor", xpath_context);
	max=xmlXPathNodeSetGetLength(xpath_object->nodesetval);
	g_return_val_if_fail(max>0, NULL);
	retval->authors=NULL;
	for(i=0 ; i<max; i++) {
		char* id;
		node=xmlXPathNodeSetItem(xpath_object->nodesetval, i);
		id=g_strdup((char *) xmlGetProp(node, "id"));
		g_return_val_if_fail(id, NULL);
		retval->authors=g_list_append(retval->authors, id);
	}
	xmlXPathFreeObject(xpath_object);
	
	xpath_object = xmlXPathEval ("/TblFd/TblFdVersion", xpath_context);
	max=xmlXPathNodeSetGetLength(xpath_object->nodesetval);
	g_return_val_if_fail(max>0, NULL);
	node=xmlXPathNodeSetItem(xpath_object->nodesetval, 0);
	version=(char*)xmlNodeGetContent(node);
	retval->version=atoi(version);
	xmlXPathFreeObject(xpath_object);

#define STRING_VALUE(xpathS, varC) 				\
	xpath_object = xmlXPathEval (xpathS, xpath_context); 	\
	max=xmlXPathNodeSetGetLength(xpath_object->nodesetval);	\
	g_return_val_if_fail(max>0, NULL);			\
	node=xmlXPathNodeSetItem(xpath_object->nodesetval, 0);	\
	varC=g_strdup((char *) xmlNodeGetContent(node));	\
	xmlXPathFreeObject(xpath_object);			

	STRING_VALUE("/TblFd/TblFdYear", retval->year)
	STRING_VALUE("/TblFd/TblFdLicense", retval->license)
	STRING_VALUE("/TblFd/TblFdApplication", retval->application)
	STRING_VALUE("/TblFd/TblFdInfo", retval->info)
#undef STRING_VALUE
	
	xmlXPathFreeContext(xpath_context);
	xmlFreeDoc(doc);

	return (GObject*) retval;
}

