/* Jentchina -XFiltersTreeReader - Xml reader
 * STRASSER Michel - 20040304		init
 * STRASSER Michel - 20040317		adding class name in toString exportation
 * */

#include "JnaXFiltersTreeReader.h"
#include "JnaXCommon.h"
#include <glib.h>
#include <glib-object.h>
#include <libxml/parser.h>
#include <libxml/tree.h>
#include <libxml/xpath.h>
#include <string.h>

static GObjectClass *jnaXFiltersTreeReader_parentClass;

static void
impl_JnaXFiltersTreeReader_freeNodeData		(GNode* node,
						 gpointer data)
{
	g_free(node->data);
}

static void
impl_JnaXFiltersTreeReader_finalize(GObject *object)
{
	JnaXFiltersTreeReader *self=(JnaXFiltersTreeReader*)object;

	g_node_children_foreach (self->root, G_TRAVERSE_ALL, impl_JnaXFiltersTreeReader_freeNodeData, NULL);
	g_node_destroy(self->root);
	
	jnaXFiltersTreeReader_parentClass->finalize(object);
}

static void
JnaXFiltersTreeReaderClass_init (JnaXFiltersTreeReaderClass *selfClass)
{
	GObjectClass *objectClass = (GObjectClass *)selfClass;
	jnaXFiltersTreeReader_parentClass = g_type_class_peek_parent (selfClass);

	objectClass->finalize=impl_JnaXFiltersTreeReader_finalize;
}

static int
impl_JnaXFiltersTreeReader_allNodesIdStrLength	(GNode* n, int depth)
{
	int retval=0;
	GNode* c;

	retval+=strlen(n->data) + depth + 1;

	for(c=n->children; c ; c=c->next)
		retval+=impl_JnaXFiltersTreeReader_allNodesIdStrLength(c, depth + 1);
	
	return retval;
}

static void
impl_JnaXFiltersTreeReader_fillStr		(GNode* n, int depth, char** str)
{
	int l, i;
	GNode* c;

	l=strlen(n->data);
	
	for(i=0; i<depth; i++) {
		(*str)[i]=' ';
	}
	
	*str+=depth;
	
	g_memmove(*str, n->data, l);
	(*str)[l]='\n';
	*str+=l+1;
	
	for(c=n->children; c ; c=c->next)
		impl_JnaXFiltersTreeReader_fillStr(c, depth + 1, str);
}


char*
JnaXFiltersTreeReader_toString		(JnaXFiltersTreeReader* self)
{
	char *retval, *str;
	int l;
	int p;
	char pS[]="Jna::XmlReaders::FiltersTreeReader\n";
	
	g_return_val_if_fail(JNA_IS_X_FILTERS_TREE_READER(self), NULL);

	p=strlen(pS);
	l=impl_JnaXFiltersTreeReader_allNodesIdStrLength(self->root, 0) + 1; // \n on each line, \0 at the end
	retval=str=g_new(char, l+p);
	g_memmove(str, pS, p);
	str+=p;
	impl_JnaXFiltersTreeReader_fillStr(self->root, 0, &str); // \n on each line, \0 at the end
	retval[l+p]=0;

	return retval;
}

void
JnaXFiltersTreeReader_print		(JnaXFiltersTreeReader* self)
{
	char* str;
	
	g_return_if_fail(JNA_IS_X_FILTERS_TREE_READER(self));
	
	str=JnaXFiltersTreeReader_toString(self);
	g_print(str);
	g_free(str);
}

GType		JnaXFiltersTreeReader_getType		(void)
{
	static GType type = 0;

	if (!type) {
		GTypeInfo info = {
			sizeof (JnaXFiltersTreeReaderClass),
			NULL, NULL,
			(GClassInitFunc) JnaXFiltersTreeReaderClass_init,
			NULL, NULL,
			sizeof (JnaXFiltersTreeReader),
			0,
			NULL, NULL
		};

		type = g_type_register_static (
			G_TYPE_OBJECT,
			"JnaXFiltersTreeReader", &info, 0);
	}

	return type;
}

static void
impl_JnaXFiltersTreeReader_buildTree		(xmlNodePtr curX, 
						 GNode* curG)
{
	xmlNodePtr children;

	children=curX->children;

	while(children) {
		int type=0;
		/* 0 : nothing
		 * 1 : filter
		 * 2 : family
		 */
		
		if(!g_strcasecmp(children->name, "TblFtFilter")) 
			type=1;
		else  if(!g_strcasecmp(children->name, "TblFtFamily")) 
			type=2;

		if(type) {
			GNode* new;
			char* id;

			id=xmlGetProp(children, "id");
			new=g_node_new(g_strdup(id));
			new=g_node_append(curG, new);

			if(type==1) { // filter
			}
			else {	// family
				impl_JnaXFiltersTreeReader_buildTree(children, new);
			}
		}
		children=children->next;
	}
}

GObject* 	
JnaXFiltersTreeReader_new			(char* xmlFile)
{
	JnaXFiltersTreeReader* retval;
	xmlDocPtr doc;
	xmlXPathContextPtr xpath_context;
	xmlXPathObjectPtr xpath_object=NULL;
	xmlNodePtr node;
	int max;

	g_return_val_if_fail(xmlFile, NULL);
	
	doc=xmlParseFile(xmlFile);

	if(!doc) {
		g_critical("File %s is not a valid xml file", xmlFile);
		return NULL;
	}

	retval=g_object_new(JNA_TYPE_X_FILTERS_TREE_READER, NULL);

	xpath_context = xmlXPathNewContext (doc);

	xpath_object = xmlXPathEval ("/TblFt", xpath_context);
	max=xmlXPathNodeSetGetLength(xpath_object->nodesetval);
	g_return_val_if_fail(max>0, NULL);
	node=xmlXPathNodeSetItem(xpath_object->nodesetval, 0);
	
	retval->root=g_node_new(strdup("Root"));
	impl_JnaXFiltersTreeReader_buildTree(node, retval->root);

	xmlXPathFreeObject(xpath_object);

	xmlXPathFreeContext(xpath_context);
	xmlFreeDoc(doc);

	return (GObject*) retval;
}

