/* Jentchina -XServersListReader
 * STRASSER Michel - 20040206		init
 * */


#include "JnaXServersListReader.h"
#include "JnaXServer.h"
#include <glib.h>
#include <glib-object.h>
#include <libxml/parser.h>
#include <libxml/tree.h>
#include <libxml/xpath.h>
#include <string.h>

static GObjectClass *jnaXServersListReader_parentClass;

static void
impl_JnaXServersListReader_finalize(GObject *object)
{
	GList* l;
	JnaXServersListReader *self=(JnaXServersListReader*)object;

	for(l=self->servers ; l ; l=g_list_next(l)) {
		g_object_unref(G_OBJECT(l->data));
	}

	g_list_free(self->servers);
	self->servers=NULL;
		
	for(l=self->serversList ; l ; l=g_list_next(l)) {
		g_free((char *)(l->data));
	}


	if(self->serversList)
		g_list_free(self->serversList);
	self->serversList=NULL;
	
	jnaXServersListReader_parentClass->finalize(object);
}

static void
JnaXServersListReaderClass_init (JnaXServersListReaderClass *selfClass)
{
	GObjectClass *objectClass = (GObjectClass *)selfClass;
	jnaXServersListReader_parentClass = g_type_class_peek_parent (selfClass);

	objectClass->finalize=impl_JnaXServersListReader_finalize;
}

char*
JnaXServersListReader_toString		(JnaXServersListReader* self)
{
	char* retval;
	GList* l;
	int length=0;
	int offset;
	
	g_return_val_if_fail(JNA_IS_X_SERVERS_LIST_READER(self), NULL);

	length=1; // terminal '\0' character
	for(l=self->servers ; l ; l=g_list_next(l)) {
		length+=7; // strlen("(, ) ; "
		length+=strlen(JNA_X_SERVER(l->data)->strategy);
		length+=strlen(JNA_X_SERVER(l->data)->opt);	
	}
	
	retval=g_new(char, length);

	offset=0;
	for(l=self->servers ; l ; l=g_list_next(l)) {
		int le;
		retval[offset]='(';
		offset++;
		le=strlen(JNA_X_SERVER(l->data)->strategy);
		g_memmove(retval+offset, JNA_X_SERVER(l->data)->strategy, le);
		offset+=le;
		retval[offset]=',';
		offset++;
		retval[offset]=' ';
		offset++;
		le=strlen(JNA_X_SERVER(l->data)->opt);
		g_memmove(retval+offset, JNA_X_SERVER(l->data)->opt, le);
		offset+=le;
		retval[offset]=')';
		offset++;
		retval[offset]=' ';
		offset++;
		retval[offset]=';';
		offset++;
		retval[offset]=' ';
		offset++;
	}
	retval[offset]=0;
	
	return retval;
}

void
JnaXServersListReader_print		(JnaXServersListReader* self)
{
	char* str;
	
	g_return_if_fail(JNA_IS_X_SERVERS_LIST_READER(self));
	
	g_print("Jna::XmlReaders::ServersListReader ");
	str=JnaXServersListReader_toString(self);
	g_print(str);
	g_free(str);
}

GType		JnaXServersListReader_getType		(void)
{
	static GType type = 0;

	if (!type) {
		GTypeInfo info = {
			sizeof (JnaXServersListReaderClass),
			NULL, NULL,
			(GClassInitFunc) JnaXServersListReaderClass_init,
			NULL, NULL,
			sizeof (JnaXServersListReader),
			0,
			NULL, NULL
		};

		type = g_type_register_static (
			G_TYPE_OBJECT,
			"JnaXServersListReader", &info, 0);
	}

	return type;
}

static GObject* 	
impl_JnaXServersListReader_newFromDoc		(xmlDocPtr doc)
{
	JnaXServersListReader* retval;
	xmlXPathContextPtr xpath_context;
	xmlXPathObjectPtr xpath_object=NULL;
	int i, max;

	xpath_context = xmlXPathNewContext (doc);
	xpath_object = xmlXPathEval ("/TblSl/TblSlServer", xpath_context);
	max=xmlXPathNodeSetGetLength(xpath_object->nodesetval);

	retval=g_object_new(JNA_TYPE_X_SERVERS_LIST_READER, NULL);
	retval->servers=NULL;

	for(i=0 ; i < max ; i++) {
		GObject* server;
		xmlNodePtr node;
		char* opt;
		char* strategy;
		
		node=xmlXPathNodeSetItem(xpath_object->nodesetval, i);
		opt=(char *) xmlNodeGetContent(node);
		strategy=(char *)xmlGetProp(node, "strategy");
		server=JnaXServer_new(strategy, opt);
		g_object_ref(server);

		retval->servers=g_list_append(retval->servers, server);
	}

	xmlXPathFreeObject(xpath_object);


	xpath_object = xmlXPathEval ("/TblSl/TblSlList", xpath_context);
	max=xmlXPathNodeSetGetLength(xpath_object->nodesetval);

	for(i=0 ; i < max ; i++) {
		xmlNodePtr node;
		char* href;
		
		node=xmlXPathNodeSetItem(xpath_object->nodesetval, i);
		href=(char *)xmlGetProp(node, "href");

		retval->serversList=g_list_append(retval->serversList, g_strdup(href));
	}

	xmlXPathFreeObject(xpath_object);

	xmlXPathFreeContext(xpath_context);

	return (GObject*) retval;
}

GObject* 	
JnaXServersListReader_newFromStream		(char* xmlStream)
{
	GObject* retval;
	xmlDocPtr doc;

	g_return_val_if_fail(xmlStream, NULL);
	
	doc=xmlParseMemory(xmlStream, strlen(xmlStream));

	if(!doc) {
		g_critical("Stream '%s' is not a valid xml stream", xmlStream);
		return NULL;
	}

	retval=impl_JnaXServersListReader_newFromDoc(doc);

	xmlFreeDoc(doc);

	return retval;
}

GObject* 	
JnaXServersListReader_new			(char* xmlFile)
{
	GObject* retval;
	xmlDocPtr doc;

	g_return_val_if_fail(xmlFile, NULL);
	
	doc=xmlParseFile(xmlFile);

	if(!doc) {
		g_critical("File %s is not a valid xml file", xmlFile);
		return NULL;
	}

	retval=impl_JnaXServersListReader_newFromDoc(doc);

	xmlFreeDoc(doc);

	return retval;
}

